/*
  Rhino test framework

   Copyright (C) 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import java.io.InputStream;
import java.io.InputStreamReader;

import javax.script.Invocable;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

/**
 * Test cases which check if ScriptExceptionTest is thrown when necesarry and if
 * this exception has all required information.
 * 
 * @author Pavel Tisnovsky
 */
public class ScriptExceptionTest extends BaseRhinoTest {
	/**
	 * Encoding used in JavaScript files.
	 */
    private static final String SCRIPT_DEFAULT_ENCODING = "UTF-8";

    ScriptEngineManager engineManager;
    ScriptEngine scriptEngine;
    Invocable invocableEngine;

    @Override
    protected void setUp(String[] args) {
        this.engineManager = new ScriptEngineManager();
        this.scriptEngine = this.engineManager.getEngineByName("JavaScript");
        this.invocableEngine = (Invocable)this.scriptEngine;
    }

    @Override
    protected void tearDown() {
        // empty block
        return;
    }

    /**
     * Check if ScriptException is thrown in case of syntax error in JavaScript
     * source.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestCommonSyntaxError() throws Exception {
        try {
            this.scriptEngine.eval("...");
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if ScriptException is thrown when unknown function is called from
     * JavaScript.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestUnknownFunction() throws Exception {
        try {
            this.scriptEngine.eval("_unknown_function_('\tHello world!')");
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if ScriptException is thrown when there's syntax error in function
     * parameters.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestSyntaxErrorArguments1() throws Exception {
        try {
            this.scriptEngine.eval("println(a'\tHello world!')");
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if ScriptException is thrown when there's syntax error in function
     * parameters.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestSyntaxErrorArguments2() throws Exception {
        try {
            this.scriptEngine.eval("println('\tHello world!',)");
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if ScriptException is thrown when there's syntax error in function
     * parameters.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestSyntaxErrorArguments3() throws Exception {
        try {
            this.scriptEngine.eval("println(,)");
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Test if ScriptException is thrown in case of syntax error in an
     * expression.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestSyntaxErrorExpression1() throws Exception {
        try {
            this.scriptEngine.eval("println(1-*2);");
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Test if ScriptException is thrown in case of syntax error in an
     * expression.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestSyntaxErrorExpression2() throws Exception {
        try {
            this.scriptEngine.eval("println(1**2);");
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Test if ScriptException is thrown in case of syntax error in an
     * expression.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestSyntaxErrorExpression3() throws Exception {
        try {
            this.scriptEngine.eval("println(>>2);");
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Test if ScriptException is thrown in case of bad command.
     *
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestBadCommand() throws Exception {
        try {
            this.scriptEngine.eval("let x=y;");
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if ScriptException is thrown when there's syntax error in function
     * parameters. Script is read from external file.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestErrorInExternalScript1() throws Exception {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("scripts/test_unknown_function.js");
            this.scriptEngine.eval(new InputStreamReader(inputStream, SCRIPT_DEFAULT_ENCODING));
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if ScriptException is thrown when there's syntax error in external
     * script.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestErrorInExternalScript2() throws Exception {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("scripts/test_invalid_parameter.js");
            this.scriptEngine.eval(new InputStreamReader(inputStream, SCRIPT_DEFAULT_ENCODING));
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if ScriptException is thrown when there's syntax error in external
     * script.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestErrorInExternalScript3() throws Exception {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("scripts/test_invalid_command_1.js");
            this.scriptEngine.eval(new InputStreamReader(inputStream, SCRIPT_DEFAULT_ENCODING));
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if ScriptException is thrown when there's syntax error in external
     * script.
     * 
     * @throws Exception
     *             thrown if script is evaluated without throwing
     *             ScriptException.
     */
    protected void testBasicScriptExceptionTestErrorInExternalScript4() throws Exception {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("scripts/test_invalid_parameter.js");
            this.scriptEngine.eval(new InputStreamReader(inputStream, SCRIPT_DEFAULT_ENCODING));
        }
        catch (ScriptException e) {
        	System.out.println("\tException thrown as expected " + e.getMessage());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if correct line number is returned when there's syntax error in
     * JavaScript.
     * 
     * @throws Exception
     *             when no ScriptException is thrown or wrong line number is
     *             returned.
     */
    protected void testScriptExceptionTestLineNumber1() throws Exception {
        try {
            this.scriptEngine.eval("_unknown_function_('\tHello world!')");
        }
        catch (ScriptException e) {
            assertEquals(1, e.getLineNumber(), "wrong line number returned " + e.getLineNumber());
            return;
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if correct line number is returned when there's syntax error in
     * JavaScript.
     * 
     * @throws Exception
     *             when no ScriptException is thrown or wrong line number is
     *             returned.
     */
    protected void testScriptExceptionTestLineNumber2() throws Exception {
        final String script =
            "println('\tok');\n" +
            "println('\tok');\n" +
            "_unknown_function_('\tHello world!');";
        try {
            this.scriptEngine.eval(script);
        }
        catch (ScriptException e) {
            assertEquals(3, e.getLineNumber(), "wrong line number returned " + e.getLineNumber());
            return;
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if correct line number is returned when there's syntax error in
     * JavaScript.
     * 
     * @throws Exception
     *             when no ScriptException is thrown or wrong line number is
     *             returned.
     */
    protected void testScriptExceptionTestLineNumber3() throws Exception {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("scripts/test_unknown_function.js");
            this.scriptEngine.eval(new InputStreamReader(inputStream, SCRIPT_DEFAULT_ENCODING));
        }
        catch (ScriptException e) {
            assertEquals(1, e.getLineNumber(), "wrong line number returned " + e.getLineNumber());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if correct line number is returned when there's syntax error in
     * JavaScript.
     * 
     * @throws Exception
     *             when no ScriptException is thrown or wrong line number is
     *             returned.
     */
    protected void testScriptExceptionTestLineNumber4() throws Exception {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("scripts/test_invalid_parameter.js");
            this.scriptEngine.eval(new InputStreamReader(inputStream, SCRIPT_DEFAULT_ENCODING));
        }
        catch (ScriptException e) {
            assertEquals(1, e.getLineNumber(), "wrong line number returned " + e.getLineNumber());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if correct line number is returned when there's syntax error in
     * JavaScript.
     * 
     * @throws Exception
     *             when no ScriptException is thrown or wrong line number is
     *             returned.
     */
    protected void testScriptExceptionTestLineNumber5() throws Exception {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("scripts/test_invalid_command_1.js");
            this.scriptEngine.eval(new InputStreamReader(inputStream, SCRIPT_DEFAULT_ENCODING));
        }
        catch (ScriptException e) {
            assertEquals(1, e.getLineNumber(), "wrong line number returned " + e.getLineNumber());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if correct line number is returned when there's syntax error in
     * JavaScript.
     * 
     * @throws Exception
     *             when no ScriptException is thrown or wrong line number is
     *             returned.
     */
    protected void testScriptExceptionTestLineNumber6() throws Exception {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("scripts/test_invalid_command_2.js");
            this.scriptEngine.eval(new InputStreamReader(inputStream, SCRIPT_DEFAULT_ENCODING));
        }
        catch (ScriptException e) {
            assertEquals(3, e.getLineNumber(), "wrong line number returned " + e.getLineNumber());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Check if correct line number is returned when there's syntax error in
     * JavaScript.
     * 
     * @throws Exception
     *             when no ScriptException is thrown or wrong line number is
     *             returned.
     */
    protected void testScriptExceptionTestLineNumber7() throws Exception {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("scripts/test_invalid_command_3.js");
            this.scriptEngine.eval(new InputStreamReader(inputStream, SCRIPT_DEFAULT_ENCODING));
        }
        catch (ScriptException e) {
            assertEquals(3, e.getLineNumber(), "wrong line number returned " + e.getLineNumber());
            return; // ok, it's correct if this exception is thrown
        }
        throw new Exception("ScriptException not thrown as expected");
    }

    /**
     * Entry point to this test case.
     *
     * @param args parameters passed from command line
     */
    public static void main(String[] args) {
        new ScriptExceptionTest().doTests(args);
    }

}
